@@ -11,6 +11,7 @@ use std::time::Duration;
 
 pub use kqueue_sys::constants::*;
 
+/// The watched object that fired the event
 #[derive(Debug, Eq, Clone)]
 pub enum Ident {
     Filename(RawFd, String),
@@ -20,6 +21,7 @@ pub enum Ident {
     Timer(i32),
 }
 
+#[doc(hidden)]
 #[derive(Debug, PartialEq, Clone)]
 pub struct Watched {
     filter: EventFilter,
@@ -27,6 +29,15 @@ pub struct Watched {
     ident: Ident,
 }
 
+/// Watches one or more resources
+///
+/// These can be created with `Watcher::new()`. You can create as many
+/// `Watcher`s as you want, and they can watch as many objects as you wish.
+/// The objects do not need to be the same type.
+///
+/// Each `Watcher` is backed by a `kqueue(2)` queue. These resources are freed
+/// on the `Watcher`s destruction. If the destructor cannot run for whatever
+/// reason, the underlying kernel object will be leaked.
 #[derive(Debug)]
 pub struct Watcher {
     watched: Vec<Watched>,
@@ -35,43 +46,107 @@ pub struct Watcher {
     opts: KqueueOpts,
 }
 
+/// Vnode events
+///
+/// These are OS-specific, and may not all be supported on your platform. Check
+/// `kqueue(2)` for more information.
 #[derive(Debug)]
 pub enum Vnode {
+    /// The file was deleted
     Delete,
+
+    /// The file received a write
     Write,
+
+    /// The file was extended with `truncate(2)`
     Extend,
+
+    /// The file was shrunk with `truncate(2)`
     Truncate,
+
+    /// The attributes of the file were changed
     Attrib,
+
+    /// The link count of the file was changed
     Link,
+
+    /// The file was renamed
     Rename,
+
+    /// Access to the file was revoked with `revoke(2)` or the fs was unmounted
     Revoke,
 }
 
+/// Process events
+///
+/// These are OS-specific, and may not all be supported on your platform. Check
+/// `kqueue(2)` for more information.
 #[derive(Debug)]
 pub enum Proc {
+    /// The watched process exited with the returned exit code
     Exit(usize),
+
+    /// The process called `fork(2)`
     Fork,
+
+    /// The process called `exec(2)`
     Exec,
+
+    /// The process called `fork(2)`, and returned the child pid.
     Track(libc::pid_t),
+
+    /// The process called `fork(2)`, but we were not able to track the child
     Trackerr,
+
+    /// The process called `fork(2)`, and returned the child pid.
+    // TODO: this is FreeBSD-specific. We can probably convert this to `Track`.
     Child(libc::pid_t),
 }
 
 // These need to be OS specific
+/// Event-specific data returned with the event.
+///
+/// Like much of this library, this is OS-specific. Check `kqueue(2)` for more
+/// details on your target OS.
 #[derive(Debug)]
 pub enum EventData {
+    /// Data relating to `Vnode` events
     Vnode(Vnode),
+
+    /// Data relating to process events
     Proc(Proc),
+
+    /// The returned number of bytes are ready for reading from the watched
+    /// descriptor
     ReadReady(usize),
+
+    /// The file is ready for writing. On some files (like sockets, pipes, etc),
+    /// the number of bytes in the write buffer will be returned.
     WriteReady(usize),
+
+    /// One of the watched signals fired. The number of times this signal was received
+    /// is returned.
     Signal(usize),
+
+    /// One of the watched timers fired. The number of times this timer fired
+    /// is returned.
     Timer(usize),
+
+    /// Some error was received
     Error(Error),
 }
 
+/// An event from a `Watcher` object.
+///
+/// An event contains both the a signifier of the watched object that triggered
+/// the event, as well as any event-specific. See the `EventData` enum for info
+/// on what event-specific data is returned for each event.
 #[derive(Debug)]
 pub struct Event {
+    /// The watched resource that triggered the event
     pub ident: Ident,
+
+    /// Any event-specific data returned with the event.
     pub data: EventData,
 }
 
@@ -79,12 +154,17 @@ pub struct EventIter<'a> {
     watcher: &'a Watcher,
 }
 
+/// Options for a `Watcher`
 #[derive(Debug)]
 pub struct KqueueOpts {
+    /// Clear state on watched objects
     clear: bool,
 }
 
 impl Default for KqueueOpts {
+    /// Returns the default options for a `Watcher`
+    ///
+    /// `clear` is set to `true`
     fn default() -> KqueueOpts {
         KqueueOpts { clear: true }
     }
@@ -106,7 +186,7 @@ impl PartialEq<Ident> for Ident {
     fn eq(&self, other: &Ident) -> bool {
         match *self {
             Ident::Filename(_, ref name) => {
-                if let &Ident::Filename(_, ref othername) = other {
+                if let Ident::Filename(_, ref othername) = *other {
                     name == othername
                 } else {
                     false
@@ -130,6 +210,10 @@ impl Ident {
 }
 
 impl Watcher {
+    /// Creates a new `Watcher`
+    ///
+    /// Creates a brand new `Watcher` with `KqueueOpts::default()`. Will return
+    /// an `io::Error` if creation fails.
     pub fn new() -> Result<Watcher> {
         let queue = unsafe { kqueue() };
 
@@ -145,11 +229,14 @@ impl Watcher {
         }
     }
 
+    /// Disables the `clear` flag on a `Watcher`. New events will no longer
+    /// be added with the `EV_CLEAR` flag on `watch`.
     pub fn disable_clears(&mut self) -> &mut Self {
         self.opts.clear = false;
         self
     }
 
+    /// Adds a `pid` to the `Watcher` to be watched
     pub fn add_pid(
         &mut self,
         pid: libc::pid_t,
@@ -169,6 +256,15 @@ impl Watcher {
         Ok(())
     }
 
+    /// Adds a file by filename to be watched
+    ///
+    /// **NB**: `kqueue(2)` is an `fd`-based API. If you add a filename with
+    /// `add_filename`, internally we open it and pass the file descriptor to
+    /// `kqueue(2)`. If the file is moved or deleted, and a new file is created
+    /// with the same name, you will not receive new events for it without
+    /// calling `add_filename` again.
+    ///
+    /// TODO: Adding new files requires calling `Watcher.watch` again
     pub fn add_filename<P: AsRef<Path>>(
         &mut self,
         filename: P,
@@ -192,6 +288,10 @@ impl Watcher {
         Ok(())
     }
 
+    /// Adds a descriptor to a `Watcher`. This or `add_file` is the preferred
+    /// way to watch a file
+    ///
+    /// TODO: Adding new files requires calling `Watcher.watch` again
     pub fn add_fd(&mut self, fd: RawFd, filter: EventFilter, flags: FilterFlag) -> Result<()> {
         let watch = Watched {
             filter,
@@ -206,20 +306,22 @@ impl Watcher {
         Ok(())
     }
 
+    /// Adds a `File` to a `Watcher`. This, or `add_fd` is the preferred way
+    /// to watch a file
+    ///
+    /// TODO: Adding new files requires calling `Watcher.watch` again
     pub fn add_file(&mut self, file: &File, filter: EventFilter, flags: FilterFlag) -> Result<()> {
         self.add_fd(file.as_raw_fd(), filter, flags)
     }
 
     fn delete_kevents(&self, ident: Ident, filter: EventFilter) -> Result<()> {
         let mut kev: Vec<kevent> = Vec::with_capacity(1);
-        kev.push(kevent {
-            ident: ident.as_usize(),
+        kev.push(kevent::new(
+            ident.as_usize(),
             filter,
-            flags: EventFlag::EV_DELETE,
-            fflags: FilterFlag::empty(),
-            data: 0,
-            udata: ptr::null_mut(),
-        });
+            EventFlag::EV_DELETE,
+            FilterFlag::empty(),
+        ));
 
         let ret = unsafe {
             kevent(
@@ -238,6 +340,7 @@ impl Watcher {
         }
     }
 
+    /// Removes a pid from a `Watcher`
     pub fn remove_pid(&mut self, pid: libc::pid_t, filter: EventFilter) -> Result<()> {
         let new_watched = self
             .watched
@@ -255,6 +358,10 @@ impl Watcher {
         self.delete_kevents(Ident::Pid(pid), filter)
     }
 
+    /// Removes a filename from a `Watcher`.
+    ///
+    /// *NB*: This matches the `filename` that this item was initially added under.
+    /// If a file has been moved, it will not be removable by the new name.
     pub fn remove_filename<P: AsRef<Path>>(
         &mut self,
         filename: P,
@@ -282,6 +389,7 @@ impl Watcher {
         self.delete_kevents(Ident::Fd(fd), filter)
     }
 
+    /// Removes an fd from a `Watcher`
     pub fn remove_fd(&mut self, fd: RawFd, filter: EventFilter) -> Result<()> {
         let new_watched = self
             .watched
@@ -299,10 +407,14 @@ impl Watcher {
         self.delete_kevents(Ident::Fd(fd), filter)
     }
 
+    /// Removes a `File` from a `Watcher`
     pub fn remove_file(&mut self, file: &File, filter: EventFilter) -> Result<()> {
         self.remove_fd(file.as_raw_fd(), filter)
     }
 
+    /// Starts watching for events from `kqueue(2)`. This function needs to
+    /// be called before `Watcher.iter()` or `Watcher.poll()` to actually
+    /// start listening for events.
     pub fn watch(&mut self) -> Result<()> {
         let mut kevs: Vec<kevent> = Vec::new();
 
@@ -315,18 +427,16 @@ impl Watcher {
                 Ident::Timer(ident) => ident as uintptr_t,
             };
 
-            kevs.push(kevent {
-                ident: raw_ident,
-                filter: watched.filter,
-                flags: if self.opts.clear {
+            kevs.push(kevent::new(
+                raw_ident,
+                watched.filter,
+                if self.opts.clear {
                     EventFlag::EV_ADD | EventFlag::EV_CLEAR
                 } else {
                     EventFlag::EV_ADD
                 },
-                fflags: watched.flags,
-                data: 0,
-                udata: ptr::null_mut(),
-            });
+                watched.flags,
+            ));
         }
 
         let ret = unsafe {
@@ -347,6 +457,8 @@ impl Watcher {
         }
     }
 
+    /// Polls for a new event, with an optional timeout. If no `timeout`
+    /// is passed, then it will block forever.
     pub fn poll(&self, timeout: Option<Duration>) -> Option<Event> {
         // poll will not block indefinitely
         // None -> return immediately
@@ -356,6 +468,8 @@ impl Watcher {
         }
     }
 
+    /// Creates an iterator that iterates over the queue. This iterator will block
+    /// until a new event is received.
     pub fn iter(&self) -> EventIter {
         EventIter { watcher: self }
     }
@@ -399,15 +513,12 @@ fn find_file_ident(watcher: &Watcher, fd: RawFd) -> Option<Ident> {
 }
 
 fn get_event(watcher: &Watcher, timeout: Option<Duration>) -> Option<Event> {
-    let mut kev = kevent {
-        ident: 0,
-        data: 0,
-        filter: EventFilter::EVFILT_SYSCOUNT,
-        fflags: FilterFlag::empty(),
-        flags: EventFlag::empty(),
-        udata: ptr::null_mut(),
-    };
-
+    let mut kev = kevent::new(
+        0,
+        EventFilter::EVFILT_SYSCOUNT,
+        EventFlag::empty(),
+        FilterFlag::empty(),
+    );
     let ret = if let Some(ts) = timeout {
         unsafe {
             kevent(
@@ -436,6 +547,7 @@ fn get_event(watcher: &Watcher, timeout: Option<Duration>) -> Option<Event> {
 // OS specific
 // TODO: Events can have more than one filter flag
 impl Event {
+    #[doc(hidden)]
     pub fn new(ev: kevent, watcher: &Watcher) -> Event {
         let data = match ev.filter {
             EventFilter::EVFILT_READ => EventData::ReadReady(ev.data as usize),
@@ -496,6 +608,7 @@ impl Event {
         Event { ident, data }
     }
 
+    #[doc(hidden)]
     pub fn from_error(ev: kevent, watcher: &Watcher) -> Event {
         let ident = match ev.filter {
             EventFilter::EVFILT_READ => find_file_ident(watcher, ev.ident as RawFd).unwrap(),
@@ -513,6 +626,7 @@ impl Event {
         }
     }
 
+    #[doc(hidden)]
     pub fn is_err(&self) -> bool {
         match self.data {
             EventData::Error(_) => true,
